<?php

namespace Drupal\mpbx\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\Entity\Media;

/**
 * Configure MapBox settings.
 */
class MpbxSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'mpbx.settings';

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mpbx_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['access_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Access token'),
      '#default_value' => $config->get('access_token'),
      '#description' => $this->t('MapBox access token. You can get it <a href="@url" target="_blank">here</a>.', [
        '@url' => 'https://account.mapbox.com/',
      ]),
      '#description_display' => 'before',
      '#pattern' => '^[pk.].*$',
    ];

    $form['style'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Style'),
      '#default_value' => $config->get('style'),
      '#description' => $this->t('MapBox style. You can create it <a href="@url" target="_blank">here</a>.', [
        '@url' => 'https://studio.mapbox.com/',
      ]),
      '#description_display' => 'before',
      '#pattern' => '^mapbox:\/\/styles\/.*\/.*$',
    ];

    $form['coordinates'] = [
      '#type' => 'geofield_latlon',
      '#title' => $this->t('Coordinates'),
      '#default_value' => $config->get('coordinates'),
    ];

    $form['extras'] = [
      '#type' => 'details',
      '#title' => $this->t('Extras'),
    ];

    $form['extras']['zoom'] = [
      '#type' => 'number',
      '#title' => $this->t('Zoom level'),
      '#min' => 0,
      '#max' => 22,
      '#step' => 1,
      '#default_value' => $config->get('zoom'),
    ];

    $control_positions = [
      'top-left' => 'top-left',
      'top-right' => 'top-right',
      'bottom-left' => 'bottom-left',
      'bottom-right' => 'bottom-right',
    ];

    $form['extras']['options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Options'),
    ];

    $form['extras']['options']['interactive'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Make map interactive'),
      '#default_value' => $config->get('interactive'),
    ];

    $form['extras']['options']['nav'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add navigation controls'),
      '#default_value' => $config->get('nav.enabled'),
    ];

    $form['extras']['options']['nav_position'] = [
      '#type' => 'select',
      '#title' => $this->t('Navigation controls position'),
      '#options' => $control_positions,
      '#default_value' => $config->get('nav.position'),
      '#states' => [
        'visible' => [
          ':input[name="nav"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['nav_zoom'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show zoom buttons'),
      '#default_value' => $config->get('nav.zoom'),
      '#states' => [
        'visible' => [
          ':input[name="nav"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['nav_compass'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show compass button'),
      '#default_value' => $config->get('nav.compass'),
      '#states' => [
        'visible' => [
          ':input[name="nav"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['nav_pitch'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Visualize pitch'),
      '#default_value' => $config->get('nav.pitch'),
      '#states' => [
        'visible' => [
          ':input[name="nav"]' => ['checked' => TRUE],
          'and',
          ':input[name="nav_compass"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['geo'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add geolocation button'),
      '#default_value' => $config->get('geo.enabled'),
    ];

    $form['extras']['options']['geo_position'] = [
      '#type' => 'select',
      '#title' => $this->t('Geolocation button position'),
      '#options' => $control_positions,
      '#default_value' => $config->get('geo.position'),
      '#states' => [
        'visible' => [
          ':input[name="geo"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['geo_accuracy'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable high accuracy'),
      '#default_value' => $config->get('geo.accuracy'),
      '#states' => [
        'visible' => [
          ':input[name="geo"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['geo_track'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Track user location'),
      '#default_value' => $config->get('geo.track'),
      '#states' => [
        'visible' => [
          ':input[name="geo"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['geo_show'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show user location'),
      '#default_value' => $config->get('geo.show'),
      '#states' => [
        'visible' => [
          ':input[name="geo"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['geo_circle'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show accuracy circle'),
      '#default_value' => $config->get('geo.circle'),
      '#states' => [
        'visible' => [
          ':input[name="geo"]' => ['checked' => TRUE],
          'and',
          ':input[name="geo_show"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['extras']['options']['fullscreen'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add fullscreen button'),
      '#default_value' => $config->get('fullscreen.enabled'),
    ];

    $form['extras']['options']['fullscreen_position'] = [
      '#type' => 'select',
      '#title' => $this->t('Fullscreen button position'),
      '#options' => $control_positions,
      '#default_value' => $config->get('fullscreen.position'),
      '#states' => [
        'visible' => [
          ':input[name="fullscreen"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $anchor_options = [
      'center' => 'center',
      'top' => 'top',
      'bottom' => 'bottom',
      'left' => 'left',
      'right' => 'right',
      'top-left' => 'top-left',
      'top-right' => 'top-right',
      'bottom-left' => 'bottom-left',
      'bottom-right' => 'bottom-right',
    ];

    $form['marker_options'] = [
      '#type' => 'details',
      '#title' => $this->t('Marker options'),
    ];

    $form['marker_options']['marker'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Marker'),
      '#target_type' => 'media',
      '#default_value' => $config->get('marker.id') ? Media::load($config->get('marker.id')) : NULL,
      '#selection_settings' => [
        'target_bundles' => [
          'image',
        ],
      ],
    ];

    $form['marker_options']['anchor'] = [
      '#type' => 'select',
      '#title' => $this->t('Anchor'),
      '#description' => $this->t('A string indicating the part of the Marker that should be positioned closest to the coordinate set'),
      '#options' => $anchor_options,
      '#default_value' => $config->get('anchor'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    /** @var \Drupal\Core\Cache\CacheBackendInterface $cacheRender */
    $cacheRender = \Drupal::service('cache.render');

    if ($form_state->getValue('marker')) {
      $marker = Media::load($form_state->getValue('marker'));
      $marker_url = file_create_url($marker->field_media_image->entity->getFileUri());

      $this->configFactory->getEditable(static::SETTINGS)
        ->set('marker.id', $form_state->getValue('marker'))
        ->set('marker.url', $marker_url);
    }
    else {
      $this->configFactory->getEditable(static::SETTINGS)
        ->set('marker.id', NULL)
        ->set('marker.url', NULL);
    }

    $this->configFactory->getEditable(static::SETTINGS)
      ->set('access_token', $form_state->getValue('access_token'))
      ->set('style', $form_state->getValue('style'))
      ->set('coordinates', $form_state->getValue('coordinates'))
      ->set('zoom', $form_state->getValue('zoom'))
      ->set('anchor', $form_state->getValue('anchor'))
      ->set('interactive', $form_state->getValue('interactive'))
      ->set('nav.enabled', $form_state->getValue('nav'))
      ->set('nav.position', $form_state->getValue('nav_position'))
      ->set('nav.zoom', $form_state->getValue('nav_zoom'))
      ->set('nav.compass', $form_state->getValue('nav_compass'))
      ->set('nav.pitch', $form_state->getValue('nav_pitch'))
      ->set('geo.enabled', $form_state->getValue('geo'))
      ->set('geo.position', $form_state->getValue('geo_position'))
      ->set('geo.accuracy', $form_state->getValue('geo_accuracy'))
      ->set('geo.track', $form_state->getValue('geo_track'))
      ->set('geo.show', $form_state->getValue('geo_show'))
      ->set('geo.circle', $form_state->getValue('geo_circle'))
      ->set('fullscreen.enabled', $form_state->getValue('fullscreen'))
      ->set('fullscreen.position', $form_state->getValue('fullscreen_position'))
      ->save();

    $cacheRender->invalidateAll();

    parent::submitForm($form, $form_state);
  }

}
